import pyglet
import basket, items
from pubsub import Publisher

class BasePlant(items.BaseItem):
    TIME_GROW = 5
    TIME_HARVEST = 10
    def __init__(self):
        items.BaseItem.__init__(self)
        
        img = self.IMAGE
        img.anchor_x = img.width / 2.0
        img.anchor_y = img.height / 2.0
        
        self.rotation = 180
        self.State = "SEED"
        self.opacity = 0.25 * 255
        self.opacityStepFactor = (255 - self.opacity) / self.TIME_GROW
        self.growTimeLeft = self.TIME_GROW
        self.harvestTimeLeft = self.TIME_HARVEST
        
    def tick(self, dt):
        if self.State == "SEED":
            self.x += self.IMAGE.width / 2.0 + 4
            self.y += self.IMAGE.height / 2.0 + 4
            self.State = "GROWING"
        elif self.State == "GROWING":
            self.growTimeLeft -= dt
            if self.growTimeLeft > 0:
                self.opacity += dt * self.opacityStepFactor
            else:
                self.opacity = 255
                self.State = "PRODUCING"
        elif self.State == "PRODUCING":
            self.rotation += dt * 180 / self.TIME_HARVEST
            if self.rotation >= 360:
                self.rotation = 360
                self.State = "READY"
                
    def acceptItem(self, item):
        if self.State == "READY" and item == items.Hand:
            self.rotation = 180
            self.State = "PRODUCING"
            Publisher.sendMessage("item.picked", self)
    
class Tomato(BasePlant):
    NAME = "Tomato"
    SHORTCUT = "T"
    IMAGE = pyglet.resource.image("Tomato.png")

class Carrot(BasePlant):
    NAME = "Carrot"
    SHORTCUT = "C"
    IMAGE = pyglet.resource.image("Carrot.png")

